﻿using System;
using System.Collections.Generic;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Xml;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Reflection;

namespace ServiceImplementation
{
	// NetDataContractSerializer를 사용하기 위한 ServiceBehavior 특성(attribute) 구현
	[AttributeUsage(AttributeTargets.Method | AttributeTargets.Interface | AttributeTargets.Class,
		AllowMultiple = false, Inherited = false)]
	public class NetDataContractFormatAttribute : Attribute, IOperationBehavior, IContractBehavior
	{
		#region "IOperationBehavior 인터페이스 구현"

		// Binding 파라메터를 추가한다. 이 Behavior에서는 Binding 파라메터를 설정하지 않는다.
		public void AddBindingParameters(OperationDescription operationDescription, BindingParameterCollection bindingParameters)
		{
		}

		// Client 측 Behavior를 추가한다.
		public void ApplyClientBehavior(OperationDescription operationDescription, ClientOperation clientOperation)
		{
			ReplaceBehavior(operationDescription);
		}

		// 서비스 측 Dispatch Behavior를 추가한다.
		public void ApplyDispatchBehavior(OperationDescription operationDescription, DispatchOperation dispatchOperation)
		{
			ReplaceBehavior(operationDescription);
		}

		// Operation 계약에서 필요한 validation 작업을 수행한다.
		public void Validate(OperationDescription operationDescription)
		{
			// XmlSerializerFormat 특성을 통해 XmlSerialzierOperationBehavior가 추가되었는지 검사한다.
			XmlSerializerOperationBehavior behavior =
				operationDescription.Behaviors.Find<XmlSerializerOperationBehavior>();
			if (behavior != null) {
				throw new InvalidOperationException("You cannot specifiy both XmlSerializerFormat and NetDataContractFormat in the operation.");
			}
			// DataContractFormat 특성이 표시되었는가 검사한다.
			Attribute[] attrs = (Attribute[])operationDescription.SyncMethod.GetCustomAttributes(
				typeof(DataContractFormatAttribute), false);
			if (attrs.Length != 0) {
				throw new InvalidOperationException("You cannot specifiy both DataContractFormat and NetDataContractFormat in the operation.");
			}
		}

		// 디폴트 Serializer인 DataContractSerializer를 사용하도록 하는
		// DataContractSerializerOperationBehavior를 제거하고 NetDataContractSerializer를 사용하도록 하는
		// NetDataContractSerializerOperationBehavior를 추가한다.
		private static void ReplaceBehavior(OperationDescription description)
		{
			DataContractSerializerOperationBehavior behavior = 
				description.Behaviors.Find<DataContractSerializerOperationBehavior>();

			if (behavior != null && behavior.GetType() != typeof(NetDataContractSerializerOperationBehavior)) {
				description.Behaviors.Remove(behavior);
				description.Behaviors.Add(new NetDataContractSerializerOperationBehavior(description));
			}
		}
		#endregion

		#region "IContractBehavior 인터페이스 구현"

		//
		// 서비스 계약 수준에서 이 특성을 사용할 수 있도록 IContractBehavior 인터페이스를 구현한다.
		//

		// Binding 파라메터를 추가한다. 이 Behavior에서는 Binding 파라메터를 설정하지 않는다.
		public void AddBindingParameters(ContractDescription contractDescription, ServiceEndpoint endpoint, BindingParameterCollection bindingParameters)
		{
			// empty
		}

		// 클라이언트 측에서 서비스 계약이 로드될 때 호출되며 클라이언트 측 behavior를 설정한다.
		public void ApplyClientBehavior(ContractDescription contractDescription, ServiceEndpoint endpoint, ClientRuntime clientRuntime)
		{
			// 각 operation에 behavior를 추가한다.
			AddBehavior(contractDescription);
		}

		// 서비스 측에서 서비스 계약이 로드될 때 호출되며 서비스의 dispatch behavior를 설정한다.
		public void ApplyDispatchBehavior(ContractDescription contractDescription, ServiceEndpoint endpoint, DispatchRuntime dispatchRuntime)
		{
			// 각 operation에 behavior를 추가한다.
			AddBehavior(contractDescription);
		}

		public void Validate(ContractDescription contractDescription, ServiceEndpoint endpoint)
		{
			// DataContractFormat 특성이 표시되었는가 검사한다.
			Attribute[] attrs = (Attribute[])contractDescription.ContractType.GetCustomAttributes(
				typeof(DataContractFormatAttribute), false);
			if (attrs.Length != 0) {
				throw new InvalidOperationException("You cannot specifiy both DataContractFormat and NetDataContractFormat in the contract.");
			}
			// XmlSerializerFormat 특성이 표시되었는가 검사한다.
			attrs = (Attribute[])contractDescription.ContractType.GetCustomAttributes(
				typeof(XmlSerializerFormatAttribute), false);
			if (attrs.Length != 0) {
				throw new InvalidOperationException("You cannot specifiy both XmlSerializerFormat and NetDataContractFormat in the contract.");
			}
		}

		// 계약 내의 각 Operation들에 대해 NetDataContractSerializer를 사용하도록
		// NetDataContractSerializerOperationBehavior 를 추가한다.
		private static void AddBehavior(ContractDescription contractDescription)
		{
			foreach (OperationDescription od in contractDescription.Operations) {
				// 이미 XmlSerializerFormat 특성에 의해 XmlSerializerOperationBehavior가 이미
				// 추가되어 있다면 아무런 작업도 하지 않는다.
				XmlSerializerOperationBehavior behavior =
					od.Behaviors.Find<XmlSerializerOperationBehavior>();
				if (behavior == null) {
					ReplaceBehavior(od);
				}
			}
		}
		#endregion
	}

	//
	// NetDataContractSerializer를 사용하도록 하는 OperationBehavior 클래스
	// DataContractSerializerOperationBehavior에서 파생하여 CreateSerializer 메쏘드를
	// 오버라이드 한다.
	//
	public class NetDataContractSerializerOperationBehavior : DataContractSerializerOperationBehavior
	{
		public NetDataContractSerializerOperationBehavior(OperationDescription description)
			: base(description)
		{
		}

		// 직렬화에 사용할 Serializer 객체를 반환한다.
		public override XmlObjectSerializer CreateSerializer(Type type, string name, string ns, IList<Type> knownTypes)
		{
			// NetDataContractSerializer는 known type을 요구하지 않는다.
			return new NetDataContractSerializer(name, ns);
		}

		// 직렬화에 사용할 Serializer 객체를 반환한다.
		public override XmlObjectSerializer CreateSerializer(Type type, XmlDictionaryString name, XmlDictionaryString ns, IList<Type> knownTypes)
		{
			// NetDataContractSerializer는 known type을 요구하지 않는다.
			return new NetDataContractSerializer(name, ns);
		}
	}
}
