﻿using System;
using System.Data;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.IO.Compression;

namespace DataServiceApp
{
	// 서비스 계약
	[ServiceContract]
	interface IDataProvider
	{
		[OperationContract]
		DataSet GetData(int rows);
		[OperationContract]
		byte[] GetDataEx(int rows);
	}

	// 서비스 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class DataService : IDataProvider
	{
		// 데이터셋을 만들어 반환한다.
		private DataSet CreateDataSet(int rows)
		{
			DataSet ds = new DataSet();
			DataTable dt = new DataTable();
			// 컬럼 생성
			for (int i = 0; i < 20; i++) {
				DataColumn column = new DataColumn("Column" + i.ToString(), typeof(string));
				dt.Columns.Add(column);
			}
			// 레코드 생성
			for (int i = 0; i < rows; i++) {
				DataRow row = dt.NewRow();
				foreach (DataColumn column in dt.Columns) {
					row[column] = "value";
				}
				dt.Rows.Add(row);
			}
			// 테이블 추가
			ds.Tables.Add(dt);
			return ds;
		}

		// 데이터셋을 BinaryFormatter를 이용하여 직렬화하고 압축한다.
		private byte[] GetBytesFromDataSet(DataSet ds)
		{
			BinaryFormatter formatter = new BinaryFormatter();
			MemoryStream stream = new MemoryStream(64 * 1024);
			GZipStream gzipStream = new GZipStream(stream, CompressionMode.Compress, false);

			ds.RemotingFormat = SerializationFormat.Binary;
			formatter.Serialize(gzipStream, ds);
			gzipStream.Close();
			stream.Close();

			return stream.ToArray();
		}

		// 단순히 데이터 셋을 반환한다.
		public DataSet GetData(int rows)
		{
			return CreateDataSet(rows);
		}

		// 데이터 셋을 BinaryFormatter를 통해 직렬화하고 압축한 결과를 반환한다.
		public byte[] GetDataEx(int rows)
		{
			DataSet ds = CreateDataSet(rows);
			byte[] result = GetBytesFromDataSet(ds);

			Console.WriteLine("{0:#,###} Rows, Serialized DataSet size = {1:#,###,###}",
				rows, result.Length);

			return result;
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Transferring a DataSet Example...");

			// 호스트 설정
			ServiceHost host = new ServiceHost(
				typeof(DataService),
				new Uri("http://localhost/wcf/example/dataservice"));

			// 바인딩 설정
			WSHttpBinding binding1 = new WSHttpBinding();
			binding1.Security.Mode = SecurityMode.None;

			WSHttpBinding binding2 = new WSHttpBinding();
			binding2.MessageEncoding = WSMessageEncoding.Mtom;
			binding2.Security.Mode = SecurityMode.None;

			CustomBinding binding3 = new CustomBinding(
				new BinaryMessageEncodingBindingElement(),
				new HttpTransportBindingElement());

			// HTTP 기반 종점 추가
			host.AddServiceEndpoint(typeof(IDataProvider), binding1, "text");
			host.AddServiceEndpoint(typeof(IDataProvider), binding2, "mtom");
			host.AddServiceEndpoint(typeof(IDataProvider), binding3, "binary");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
