﻿using System;
using System.IO;
using System.ServiceProcess;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace FileMonitorService
{
	class ServiceMain
	{
		// Windows 서비스 메인 엔트리 포인트
		static void Main(string[] args)
		{
			FileMonSvc service = new FileMonSvc();

			// 디버그 옵션이 있는 경우 서비스 시작을 에뮬레이션 한다.
			if (args.Length > 0 && String.Compare(args[0], "/debug", true) == 0) {
				service.StartService();
				Console.WriteLine("Service start is emulated... press any key to stop service.");
				Console.ReadKey(true);
				service.StopService();
			}
			else {
				// 디버그 옵션이 없는 경우 정상적인 Windows 서비스 시작을 수행한다.
				ServiceBase.Run(service);
			}
		}
	}

	// Windows 서비스 구현
	public class FileMonSvc : ServiceBase
	{
		// 모니터링 할 파일 경로
		private string _FilePath = null;
		private FileSystemWatcher _Watcher = null;
		private DateTime _LastModified = DateTime.MinValue;
		private ServiceHost _Host = null;

		public FileMonSvc()
		{
			this.ServiceName = "SampleFileMonSvc";
		}

		// 디버깅을 위한 서비스 시작
		internal void StartService()
		{
			this.OnStart(null);
		}

		// 디버깅을 위한 서비스 중단
		internal void StopService()
		{
			this.OnStop();
		}

		// 서비스 시작 notification
		protected override void OnStart(string[] args)
		{
			// 모니터링 시작
			if (_FilePath != null) {
				StartMonitor();
			}
			// WCF 서비스 구동
			_Host = new ServiceHost(typeof(FileMonitorWCFService),
				new Uri("http://localhost/wcf/example/filemonitorservice"));
			NetNamedPipeBinding binding = new NetNamedPipeBinding();
			_Host.AddServiceEndpoint(
				typeof(IFileMonitor),
				binding,
				"net.pipe://localhost/FileMonitorService");
			// WSDL을 위한 설정
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			_Host.Description.Behaviors.Add(behavior);
			// 서비스 인스턴스 설정
			FileMonitorWCFService.WindowsServiceInstance = this;
			// WCF 서비스 리스닝 시작
			_Host.Open();
		}

		// 서비스 종료 notification
		protected override void OnStop()
		{
			_Host.Close();
			StopMonitor();
		}

		public void StartMonitor()
		{
			if (String.IsNullOrEmpty(_FilePath)) {
				this.EventLog.WriteEntry("A file to monitor is not specified.",
					EventLogEntryType.Warning);
				return;
			}
			if (File.Exists(_FilePath) == false) {
				this.EventLog.WriteEntry("Cannot find the file to monitor.\n\n" + _FilePath, 
					EventLogEntryType.Warning);
				return;
			}
			if (_Watcher != null) {
				Cleanup();
			}
			// 주어진 파일의 쓰기에 대한 감시를 수행한다.
			string dir = Path.GetDirectoryName(_FilePath);
			string filter = Path.GetFileName(_FilePath);
			_Watcher = new FileSystemWatcher(dir, filter);
			_Watcher.NotifyFilter = NotifyFilters.LastWrite; 
			_Watcher.Changed += new FileSystemEventHandler(OnFileChanged);
			_Watcher.EnableRaisingEvents = true;
			this.EventLog.WriteEntry("Start to monitor the file,\n\n" + _FilePath);
		}

		// 모니터링을 중단한다.
		public void StopMonitor()
		{
			if (_Watcher != null) {
				Cleanup();
			}
		}

		// 모니터링에 대한 클린업 작업 수행
		private void Cleanup()
		{
			_Watcher.EnableRaisingEvents = false;
			_Watcher.Dispose();
			_Watcher = null;
		}

		// FileSystemWatcher 클래스의 Changed 이벤트 핸들러
		private void OnFileChanged(object sender, FileSystemEventArgs e)
		{
			this.EventLog.WriteEntry("The file is modified !");
			_LastModified = DateTime.Now;
		}

		// 감시할 파일 경로 설정
		internal void SetFilePath(string path)
		{
			StopMonitor();
			_FilePath = path;
			StartMonitor();
		}

		// 현재 설정된 파일 경로
		internal string FilePath
		{
			get { return _FilePath; }
		}

		// 마지막으로 변경된 시간 반환
		internal DateTime LastModified
		{
			get { return _LastModified; }
		}
	}

	// WCF 서비스 계약
	[ServiceContract]
	interface IFileMonitor
	{
		// 감시할 파일 경로 반환
		[OperationContract]
		string GetFilePath();
		// 감시할 파일 경로 설정
		[OperationContract]
		void SetFilePath(string path);
		// 가장 마지막에 변경된 날짜/시간을 반환
		[OperationContract]
		DateTime GetLastModified();
	}

	// WCF 서비스 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class FileMonitorWCFService : IFileMonitor
	{
		private static FileMonSvc _WindowsServiceInstance = null;

		//
		public string GetFilePath()
		{
			return _WindowsServiceInstance.FilePath;
		}

		public void SetFilePath(string path)
		{
			_WindowsServiceInstance.SetFilePath(path);
		}

		public DateTime GetLastModified()
		{
			return _WindowsServiceInstance.LastModified;
		}

		internal static FileMonSvc WindowsServiceInstance
		{
			get { return _WindowsServiceInstance; }
			set { _WindowsServiceInstance = value; }
		}
	}
}
