﻿using System;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace BasicClientApp
{
	#region "서비스 계약 선언"
	//
	// 다음과 같이 서비스에서 사용하는 서비스 계약 및 데이터 계약을 직접
	// 선언해 주어도 된다. 선언할 때는 서비스가 사용하는 계약과 호환만 되면 된다.
	// 따라서 인터페이스, 클래스 이름이 다르더라도 Name 속성등을 이용하여
	// 이를 맞추어 줄 수 있다.
	//
	[ServiceContract(Name="IMyContract")]
	interface IClientContract
	{
		[OperationContract]
		void MyMethod(ClientData data);
	}

	// 데이터 계약을 선언할 때 명시적으로 XML 네임스페이스를 주지 않으면 디폴트 값으로
	// http://schemas.datacontract.org/2004/07/<CLR_Namespace> 가 사용되므로
	// 서비스와 클라이언트가 서로 다른 XML 네임스페이스의 데이터 계약을 사용할 수 있다.
	// 이 경우 서비스가 데이터를 인식하지 못하므로 클라이언트 측에서
	// 이를 맞추어 주어야 한다.
	// 물론 svcutil.exe는 이와 같은 Namespace 선언을 자동으로 알아서 해준다.
	[DataContract(Name = "MyData", Namespace = "http://schemas.datacontract.org/2004/07/TestServiceApp")]
	class ClientData
	{
		[DataMember]
		public string Field;
	}
	#endregion

	#region "프록시 래퍼 클래스"

	class MyProxy : ClientBase<IClientContract>
	{
		public MyProxy() : base() { }
		public MyProxy(System.ServiceModel.Channels.Binding binding, EndpointAddress remoteAddress)
			: base(binding, remoteAddress) { }
		public MyProxy(string configurationName)
			: base(configurationName) { }

		public void MyMethod(ClientData data)
		{
			this.Channel.MyMethod(data);
		}
	}
	#endregion

	class ClientApp
	{
		static void Main(string[] args)
		{
			ClientUsingChannelFactory1();
			ClientUsingChannelFactory2();
			ClientUsingChannelFactory3();
			ClientUsingClientBase1();
			ClientUsingClientBase2();
		}

		// ChannelFactory<T>를 이용한 클라이언트 코드 예제 #1
		private static void ClientUsingChannelFactory1()
		{
			Console.WriteLine("Call the service using ChannelFactory<T> #1");

			string remoteAddress = "http://loner/wcf/example/testservice";
			BasicHttpBinding binding = new BasicHttpBinding();
			ChannelFactory<IClientContract> factory =
				new ChannelFactory<IClientContract>(binding, remoteAddress);
			IClientContract proxy = factory.CreateChannel();
			try {
				ClientData data = new ClientData();
				data.Field = "ChannelFactory Sample #1";
				proxy.MyMethod(data);
			}
			finally {
				(proxy as IDisposable).Dispose();
			}
		}

		// ChannelFactory<T>를 이용한 클라이언트 코드 예제 #2
		private static void ClientUsingChannelFactory2()
		{
			Console.WriteLine("Call the service using ChannelFactory<T> #2");

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(IClientContract)),
				new BasicHttpBinding(),
				new EndpointAddress("http://loner/wcf/example/testservice"));
			ChannelFactory<IClientContract> factory =
				new ChannelFactory<IClientContract>(ep);
			IClientContract proxy = factory.CreateChannel();
			using (proxy as IDisposable) {
				ClientData data = new ClientData();
				data.Field = "ChannelFactory Sample #2";
				proxy.MyMethod(data);
			}
		}

		// ChannelFactory<T>를 이용한 클라이언트 코드 예제 #3
		private static void ClientUsingChannelFactory3()
		{
			Console.WriteLine("Call the service using ChannelFactory<T> #3");

			ChannelFactory<IClientContract> factory =
				new ChannelFactory<IClientContract>("EndpointName");
			IClientContract proxy = factory.CreateChannel();
			using (proxy as IDisposable) {
				(proxy as IClientChannel).Open();
				if ((proxy as ICommunicationObject).State != CommunicationState.Opened) {
					throw new InvalidOperationException("Open failure...");
				}
				ClientData data = new ClientData();
				data.Field = "ChannelFactory Sample #3";
				proxy.MyMethod(data);
			}
		}

		// ClientBase<T>를 이용한 클라이언트 코드 예제 #1
		private static void ClientUsingClientBase1()
		{
			Console.WriteLine("Call the service using ClientBase<T> #1");
			using (MyProxy proxy = new MyProxy("EndpointName")) {
				ClientData data = new ClientData();
				data.Field = "ClientBase Sample #1";
				proxy.MyMethod(data);

			}
		}

		// ClientBase<T>를 이용한 클라이언트 코드 예제 #2
		private static void ClientUsingClientBase2()
		{
			Console.WriteLine("Call the service using ClientBase<T> #2");

			BasicHttpBinding binding = new BasicHttpBinding();
			EndpointAddress remoteAddress = 
				new EndpointAddress("http://loner/wcf/example/testservice");

			using (MyProxy proxy = new MyProxy(binding, remoteAddress)) {
				proxy.Open();
				if (proxy.State != CommunicationState.Opened) {
					throw new InvalidOperationException("Open failure...");
				}
				ClientData data = new ClientData();
				data.Field = "ClientBase Sample #2";
				proxy.MyMethod(data);

			}
		}
	}
}
